/****************************************************************************
*
*                SciTech Nucleus Command Line Options program
*
*               Copyright (C) 1991-1998 SciTech Software, Inc.
*                            All rights reserved.
*
*  ======================================================================
*  |REMOVAL OR MODIFICATION OF THIS HEADER IS STRICTLY PROHIBITED BY LAW|
*  |                                                                    |
*  |This copyrighted computer code contains proprietary technology      |
*  |owned by SciTech Software, Inc., located at 505 Wall Street,        |
*  |Chico, CA 95928 USA (http://www.scitechsoft.com).                   |
*  |                                                                    |
*  |The contents of this file are subject to the SciTech Nucleus        |
*  |License; you may *not* use this file or related software except in  |
*  |compliance with the License. You may obtain a copy of the License   |
*  |at http://www.scitechsoft.com/nucleus-license.txt                   |
*  |                                                                    |
*  |Software distributed under the License is distributed on an         |
*  |"AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or      |
*  |implied. See the License for the specific language governing        |
*  |rights and limitations under the License.                           |
*  |                                                                    |
*  |REMOVAL OR MODIFICATION OF THIS HEADER IS STRICTLY PROHIBITED BY LAW|
*  ======================================================================
*
* Language:     ANSI C
* Environment:  Any 32-bit protected mode environment
*
* Description:  Simple program that allows the Nucleus opt to be
*				changed from the command line.
*
****************************************************************************/

#include "nucleus/graphics.h"
#include "sdd/copyrigh.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

/*---------------------------- Global Variables ---------------------------*/

static GA_devCtx		*dc = NULL;
static GA_initFuncs		init;
static GA_configInfo	info;
static GA_options		opt;
static GA_globalOptions	gOpt;

/*----------------------------- Implementation ----------------------------*/

/****************************************************************************
REMARKS:
Display the sign on banner.
****************************************************************************/
static void banner(void)
{
	printf("GAOption - SciTech Nucleus Option Selection Program\n");
	printf("           %s\n", SCITECH_COPYRIGHT_MSG_SHORT);
	printf("\n");
}

/****************************************************************************
REMARKS:
Display the help screen.
****************************************************************************/
static void help(void)
{
	banner();
	printf("Valid commands are:\n");
	printf("\n");
	printf("  gaoption show       <device>\n");
	printf("  gaoption accel      [none | basic | most | full] <device>\n");
	printf("  gaoption vbe        [on | off]\n");
	printf("  gaoption vga        [on | off]\n");
	printf("  gaoption noncert    [on | off]\n");
	printf("  gaoption nowcomb    [on | off]\n");
	printf("  gaoption noddc      [on | off]\n");
        printf("  gaoption agpfwrite  [on | off]\n");
	printf("  gaoption lcdusebios [auto | on | off] <device>\n");
	printf("  gaoption agpmem     [memsize]\n");
	printf("  gaoption usesysmem  [memsize | off]\n");
	printf("  gaoption prefer16   [on | off] <device>\n");
	printf("  gaoption prefer32   [on | off] <device>\n");
	printf("  gaoption invert     [on | off] <device>\n");
	printf("  gaoption portrait   [on | off] <device>\n");
	printf("  gaoption flipped    [on | off] <device>\n");
	printf("  gaoption virtual    [on | off] <[xRes] [yRes] [horz count] [vert count]>\n");
	exit(-1);
}

/****************************************************************************
REMARKS:
Load the global device driver options.
****************************************************************************/
static void LoadGlobalOptions(void)
{
	gOpt.dwSize = sizeof(gOpt);
	GA_getGlobalOptions(&gOpt, false);
}

/****************************************************************************
REMARKS:
Load the Nucleus driver and intialise it.
****************************************************************************/
static void LoadDriver(
	int deviceIndex)
{
	if (!dc) {
		if ((dc = GA_loadDriver(deviceIndex,false)) == NULL)
			PM_fatalError(GA_errorMsg(GA_status()));
		init.dwSize = sizeof(init);
		if (!GA_queryFunctions(dc,GA_GET_INITFUNCS,&init))
			PM_fatalError("Unable to get device driver functions!");
		info.dwSize = sizeof(info);
		init.GetConfigInfo(&info);
		opt.dwSize = sizeof(opt);
		init.GetOptions(&opt);
		LoadGlobalOptions();
		}
}

/****************************************************************************
REMARKS:
Function to enable virtual display mode. This is a simple function that
can enable any number of displays in a rectangular layout with each
display using the same resolution.
****************************************************************************/
void EnableVirtualMode(
	int argc,
	char *argv[])
{
	int x,y,i,xRes,yRes,numX,numY;

	if (argc != 7)
		help();
	xRes = atoi(argv[3]);
	yRes = atoi(argv[4]);
	numX = atoi(argv[5]);
	numY = atoi(argv[6]);
	gOpt.virtualSize.left = gOpt.virtualSize.top = 0;
	gOpt.virtualSize.right = xRes * numX;
	gOpt.virtualSize.bottom = yRes * numY;
	memset(&gOpt.resolutions,0,sizeof(gOpt.resolutions));
	memset(&gOpt.bounds,0,sizeof(gOpt.bounds));
	for (i = x = 0; x < numX; x++) {
		for (y = 0; y < numY; y++, i++) {
			gOpt.resolutions[i].left = gOpt.resolutions[i].top = 0;
			gOpt.resolutions[i].right = xRes;
			gOpt.resolutions[i].bottom = yRes;
			gOpt.bounds[i].left = x * xRes;
			gOpt.bounds[i].top = y * yRes;
			gOpt.bounds[i].right = (x+1) * xRes;
			gOpt.bounds[i].bottom = (y+1) * yRes;
			}
		}
	gOpt.bVirtualDisplay = true;
}

/****************************************************************************
REMARKS:
Function to enable the hardware acceleration features of the drivers
depending on a sliding scale.
****************************************************************************/
void SetHardwareAccel(
	int argc,
	char *argv[])
{
	if (argc != 3 && argc != 4)
		help();
	if (stricmp(argv[2],"full") == 0) {
		opt.bHardwareCursor			= true;
		opt.bHardwareColorCursor	= true;
		opt.bHardwareVideo			= true;
		opt.bHardwareAccel2D		= true;
		opt.bHardwareAccel3D        = true;
		opt.bMonoPattern            = true;
		opt.bTransMonoPattern       = true;
		opt.bColorPattern           = true;
		opt.bTransColorPattern      = true;
		opt.bSysMem                 = true;
		opt.bLinear                 = true;
		opt.bBusMaster              = true;
		opt.bDrawScanList           = true;
		opt.bDrawEllipseList        = true;
		opt.bDrawFatEllipseList     = true;
		opt.bDrawRect               = true;
		opt.bDrawRectLin            = true;
		opt.bDrawTrap               = true;
		opt.bDrawLine               = true;
		opt.bDrawStippleLine        = true;
		opt.bPutMonoImage           = true;
		opt.bClipMonoImage          = true;
		opt.bBitBlt                 = true;
		opt.bBitBltPatt             = true;
		opt.bBitBltColorPatt        = true;
		opt.bSrcTransBlt            = true;
		opt.bDstTransBlt            = true;
		opt.bStretchBlt             = true;
		opt.bConvertBlt             = true;
		opt.bStretchConvertBlt      = true;
		opt.bBitBltFx               = true;
		opt.bGetBitmap              = true;
		opt.accelType				= gaAccelType_Full;
		}
	else if (stricmp(argv[2],"most") == 0) {
		opt.bHardwareCursor			= false;
		opt.bHardwareColorCursor	= false;
		opt.bHardwareVideo			= true;
		opt.bHardwareAccel2D		= true;
		opt.bHardwareAccel3D        = true;
		opt.bMonoPattern            = true;
		opt.bTransMonoPattern       = true;
		opt.bColorPattern           = true;
		opt.bTransColorPattern      = true;
		opt.bSysMem                 = true;
		opt.bLinear                 = true;
		opt.bBusMaster              = true;
		opt.bDrawScanList           = true;
		opt.bDrawEllipseList        = true;
		opt.bDrawFatEllipseList     = true;
		opt.bDrawRect               = true;
		opt.bDrawRectLin            = true;
		opt.bDrawTrap               = true;
		opt.bDrawLine               = true;
		opt.bDrawStippleLine        = true;
		opt.bPutMonoImage           = true;
		opt.bClipMonoImage          = true;
		opt.bBitBlt                 = true;
		opt.bBitBltPatt             = true;
		opt.bBitBltColorPatt        = true;
		opt.bSrcTransBlt            = true;
		opt.bDstTransBlt            = true;
		opt.bStretchBlt             = true;
		opt.bConvertBlt             = true;
		opt.bStretchConvertBlt      = true;
		opt.bBitBltFx               = true;
		opt.bGetBitmap              = true;
		opt.accelType				= gaAccelType_Most;
		}
	else if (stricmp(argv[2],"basic") == 0) {
		opt.bHardwareCursor			= false;
		opt.bHardwareColorCursor	= false;
		opt.bHardwareVideo			= false;
		opt.bHardwareAccel2D		= true;
		opt.bHardwareAccel3D        = false;
		opt.bMonoPattern            = true;
		opt.bTransMonoPattern       = true;
		opt.bColorPattern           = true;
		opt.bTransColorPattern      = true;
		opt.bSysMem                 = false;
		opt.bLinear                 = false;
		opt.bBusMaster              = false;
		opt.bDrawScanList           = true;
		opt.bDrawEllipseList        = true;
		opt.bDrawFatEllipseList     = true;
		opt.bDrawRect               = true;
		opt.bDrawRectLin            = false;
		opt.bDrawTrap               = true;
		opt.bDrawLine               = false;
		opt.bDrawStippleLine        = false;
		opt.bPutMonoImage           = false;
		opt.bClipMonoImage          = false;
		opt.bBitBlt                 = true;
		opt.bBitBltPatt             = false;
		opt.bBitBltColorPatt        = false;
		opt.bSrcTransBlt            = false;
		opt.bDstTransBlt            = false;
		opt.bStretchBlt             = false;
		opt.bConvertBlt             = false;
		opt.bStretchConvertBlt      = false;
		opt.bBitBltFx               = false;
		opt.bGetBitmap              = false;
		opt.accelType				= gaAccelType_Basic;
		}
	else if (stricmp(argv[2],"none") == 0) {
		opt.bHardwareCursor			= false;
		opt.bHardwareColorCursor	= false;
		opt.bHardwareVideo			= false;
		opt.bHardwareAccel2D		= false;
		opt.bHardwareAccel3D        = false;
		opt.bMonoPattern            = false;
		opt.bTransMonoPattern       = false;
		opt.bColorPattern           = false;
		opt.bTransColorPattern      = false;
		opt.bSysMem                 = false;
		opt.bLinear                 = false;
		opt.bBusMaster              = false;
		opt.bDrawScanList           = false;
		opt.bDrawEllipseList        = false;
		opt.bDrawFatEllipseList     = false;
		opt.bDrawRect               = false;
		opt.bDrawRectLin            = false;
		opt.bDrawTrap               = false;
		opt.bDrawLine               = false;
		opt.bDrawStippleLine        = false;
		opt.bPutMonoImage           = false;
		opt.bClipMonoImage          = false;
		opt.bBitBlt                 = false;
		opt.bBitBltPatt             = false;
		opt.bBitBltColorPatt        = false;
		opt.bSrcTransBlt            = false;
		opt.bDstTransBlt            = false;
		opt.bStretchBlt             = false;
		opt.bConvertBlt             = false;
		opt.bStretchConvertBlt      = false;
		opt.bBitBltFx               = false;
		opt.bGetBitmap              = false;
		opt.accelType				= gaAccelType_None;
		}
	else
		help();
}

/****************************************************************************
REMARKS:
Returns the hardware acceleration type.
****************************************************************************/
const char *GetLCDMode(void)
{
	switch (gOpt.bLCDUseBIOS) {
		case gaLCDUseBIOS_Off:	return "Off";
		case gaLCDUseBIOS_Auto:	return "Auto";
		case gaLCDUseBIOS_On:	return "On";
		}
	return "Unknown!";
}

/****************************************************************************
REMARKS:
Returns the AGP shared memory size
****************************************************************************/
const char *GetAGPMemSize(void)
{
	static char buf[20];
	sprintf(buf,"%d Kb", gOpt.dwSharedAGPMemSize);
	return buf;
}

/****************************************************************************
REMARKS:
Returns the hardware acceleration type.
****************************************************************************/
const char *GetSysMemMode(void)
{
	static char buf[20];

	if (gOpt.bUseMemoryDriver) {
		sprintf(buf,"On (%d Kb)", gOpt.wSysMemSize);
		return buf;
		}
	return "Off";
}

/****************************************************************************
REMARKS:
Show the current settings for the device.
****************************************************************************/
static void ShowGlobalSettings(void)
{
	int	i;

	LoadGlobalOptions();
	printf(
		"Global options for all devices:\n\n"
		"  Force VBE Fallback ...... %s\n"
		"  Force VGA Fallback ...... %s\n"
		"  Allow non-certified ..... %s\n"
		"  Disable write combining . %s\n"
		"  Use BIOS for LCD panel... %s\n"
		"  Shared AGP memory size... %s\n"
		"  Use system memory driver. %s\n"
                "  Disable DDC detection.... %s\n"
                "  Enable AGP FastWrite..... %s\n",
                gOpt.bVBEOnly           ? "On" : "Off",
		gOpt.bVGAOnly           ? "On" : "Off",
		gOpt.bAllowNonCertified ? "On" : "Off",
		gOpt.bNoWriteCombine    ? "On" : "Off",
		GetLCDMode(),
		GetAGPMemSize(),		
		GetSysMemMode(),
                gOpt.bNoDDCDetect    ? "On" : "Off",
                gOpt.bAGPFastWrite   ? "On" : "Off");
	if (gOpt.bVirtualDisplay) {
		printf("  Virtual Display.......... Yes\n");
		for (i = 0; i < GA_MAX_VIRTUAL_DISPLAYS; i++) {
			if (gOpt.bounds[i].right == 0)
				break;
			printf("    %2d: %4d x %4d (%4d,%4d,%4d,%4d)\n",
				i,
				gOpt.resolutions[i].right,
				gOpt.resolutions[i].bottom,
				gOpt.bounds[i].left,
				gOpt.bounds[i].top,
				gOpt.bounds[i].right,
				gOpt.bounds[i].bottom);
			}
		}
	else
		printf("  Virtual Display.......... Off\n");
}

/****************************************************************************
REMARKS:
Returns the hardware acceleration type.
****************************************************************************/
const char *GetHardwareAccelType(void)
{
	switch (opt.accelType) {
		case gaAccelType_Full:	return "Full";
		case gaAccelType_Most:	return "Most";
		case gaAccelType_Basic:	return "Basic";
		case gaAccelType_None:	return "None";
		}
	return "Unknown!";
}

/****************************************************************************
REMARKS:
Show the current settings for the device.
****************************************************************************/
static void ShowSettings(
	int deviceIndex)
{
	LoadDriver(deviceIndex);
	printf(
		"Options for %s %s (device %d):\n\n"
		"  Invert .................. %s\n"
		"  Portrait ................ %s\n"
		"  Flipped ................. %s\n"
		"  Prefer 16 bit per pixel.. %s\n"
		"  Prefer 32 bit per pixel.. %s\n"
		"  Hardware acceleration.... %s\n\n",
		info.ManufacturerName, info.ChipsetName,
		deviceIndex,
		opt.bInvertColors 	    ? "On" : "Off",
		opt.bPortrait     	    ? "On" : "Off",
		opt.bFlipped      	    ? "On" : "Off",
		opt.bPrefer16bpp		? "On" : "Off",
		opt.bPrefer32bpp		? "On" : "Off",
		GetHardwareAccelType());
}

/****************************************************************************
REMARKS:
Main program entry point
****************************************************************************/
int main(
	int argc,
	char *argv[])
{
	/* Now handle the query */
	if (argc >= 2 && stricmp(argv[1],"show") == 0) {
		if (argc == 2) {
			ShowSettings(0);
			ShowGlobalSettings();
			}
		else if (tolower(argv[2][0]) == 'g')
			ShowGlobalSettings();
		else {
			ShowSettings(atoi(argv[2]));
			ShowGlobalSettings();
			}
		}
	else if (argc >= 3) {
		ibool 	enable = stricmp(argv[2],"on") == 0;
		int 	device = (argc == 3) ? 0 : atoi(argv[3]);

		LoadGlobalOptions();
		if (stricmp(argv[1],"vbe") == 0)
			gOpt.bVBEOnly = enable;
		else if (stricmp(argv[1],"vga") == 0)
			gOpt.bVGAOnly = enable;
		else if (stricmp(argv[1],"noncert") == 0)
			gOpt.bAllowNonCertified = enable;
		else if (stricmp(argv[1],"nowcomb") == 0)
			gOpt.bNoWriteCombine = enable;
		else if (stricmp(argv[1],"noddc") == 0)
			gOpt.bNoDDCDetect = enable;
                else if (stricmp(argv[1],"agpfwrite") == 0)
                        gOpt.bAGPFastWrite = enable;
		else if (stricmp(argv[1],"virtual") == 0) {
			if (enable)
				EnableVirtualMode(argc,argv);
			else
				gOpt.bVirtualDisplay = false;
			}
		else if (stricmp(argv[1],"lcdusebios") == 0) {
			if (stricmp(argv[2],"auto") == 0)
				gOpt.bLCDUseBIOS = gaLCDUseBIOS_Auto;
			else if (enable)
				gOpt.bLCDUseBIOS = gaLCDUseBIOS_On;
			else
				gOpt.bLCDUseBIOS = gaLCDUseBIOS_Off;
			}
		else if (stricmp(argv[1],"agpmem") == 0) {
			gOpt.dwSharedAGPMemSize = atoi(argv[2]);
            if (gOpt.dwSharedAGPMemSize < 512 || gOpt.dwSharedAGPMemSize > 65536)
                gOpt.dwSharedAGPMemSize = 0;
			}
		else if (stricmp(argv[1],"usesysmem") == 0) {
			gOpt.bUseMemoryDriver = (stricmp(argv[2],"off") != 0);
			gOpt.wSysMemSize = atoi(argv[2]);
			}
		else {
			LoadDriver(device);
			if (stricmp(argv[1],"invert") == 0)
				opt.bInvertColors = enable;
			else if (stricmp(argv[1],"portrait") == 0)
				opt.bPortrait = enable;
			else if (stricmp(argv[1],"flipped") == 0)
				opt.bFlipped = enable;
			else if (stricmp(argv[1],"prefer16") == 0)
				opt.bPrefer16bpp = enable;
			else if (stricmp(argv[1],"prefer32") == 0)
				opt.bPrefer32bpp = enable;
			else if (stricmp(argv[1],"accel") == 0)
				SetHardwareAccel(argc,argv);
			init.SetOptions(&opt);
			GA_saveOptions(dc,&opt);
			ShowSettings(device);
			}
		GA_setGlobalOptions(&gOpt);
		GA_saveGlobalOptions(&gOpt);
		ShowGlobalSettings();
		}
	else
		help();
	if (dc)
		GA_unloadDriver(dc);
	return 0;
}

